'''
Contact: jingbo.liu2013@gmail.com
Converting the tree module from python to cython in order to boost efficiency
'''
from collections import Counter
import math
import numpy as np
from time import time as now
from .cyfun import search_maxgain_mask

class TreeNode(object):
    '''
    A node class for a decision tree.
    '''
    def __init__(self):
        self.column = None  # (int)    index of feature to split on
        self.value = None  # value of the feature to split on
        self.categorical = True  # (bool) whether or not node is split on
                                 #  categorial feature
        self.name = None    # (string) name of feature (or name of class in the
                            #          case of a list)
        self.left = None    # (TreeNode) left child
        self.right = None   # (TreeNode) right child
        self.leaf = False   # (bool)   true if node is a leaf, false otherwise
        self.classes = Counter()  # (Counter) only necessary for leaf node:
                                  #           key is class name and value is
                                  #           count of the count of data points
                                  #           that terminate at this leaf

    def predict_one(self, x):
        '''
        INPUT:
            - x: 1d numpy array (single data point)
        OUTPUT:
            - y: predicted label
        Return the predicted label for a single data point.
        '''
        if self.leaf:
            return self.name

        col_value = x[self.column]
        if self.categorical:
            if col_value == self.value:
                return self.left.predict_one(x)
            else:
                return self.right.predict_one(x)
        else:
            if col_value < self.value:
                return self.left.predict_one(x)
            else:
                return self.right.predict_one(x)

    # This is for visualizing your tree. You don't need to look into this code.
    def as_string(self, level=0, prefix=""):
        '''
        INPUT:
            - level: int (amount to indent)
        OUTPUT:
            - prefix: str (to start the line with)
        Return a string representation of the tree rooted at this node.
        '''
        result = ""
        if prefix:
            indent = "  |   " * (level - 1) + "  |-> "
            result += indent + prefix + "\n"
        indent = "  |   " * level
        result += indent + "  " + str(self.name) + "\n"
        if not self.leaf:
            if self.categorical:
                left_key = str(self.value)
                right_key = "no " + str(self.value)
            else:
                left_key = "< " + str(self.value)
                right_key = ">= " + str(self.value)
            result += self.left.as_string(level + 1, left_key + ":")
            result += self.right.as_string(level + 1, right_key + ":")
        return result

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __repr__(self):
        return self.as_string().strip()


class DecisionTreeClassifier(object):
    '''
    A decision tree class.
    Input to Decision tree should be either string for categorical feature or float for continuous features
    This module do not support boolean
    '''

    def __init__(self, impurity_criterion='gini', max_features=None, feature_names=None, feature_threshold=None,
                 max_depth=float('inf'), continous_variable_splits="unique", impurity_dict={}):
        '''
        Initialize an empty DecisionTree.
        '''

        self.root = None  # root Nodess
        self.feature_names = feature_names  # string names of features (for interpreting
                                   # the tree)
        self.categorical = None  # Boolean array of whether variable is
                                 # categorical (or continuous)
        self.max_features = max_features
        self.max_depth = max_depth
        self.feature_threshold_out_ = {}
        self.feature_threshold = feature_threshold
        self.impurity_criterion = self._entropy \
                                  if impurity_criterion == 'entropy' \
                                  else self._gini
        self.impurity_dict = impurity_dict
        self.continous_variable_splits = continous_variable_splits  ## Percentile of unique values
        self.feature_importance_ = None
        self.feature_selected_ = None
        self.time1 = 0  ## Time for choose split
        # self.n1 = 0     ## Number of calls choose split
        # self.time2 = 0  ## Time for make split
        # self.n2 = 0     ## Number of calls make split
        # self.time3 = 0  ## Time for information gain
        # self.n3 = 0     ## Number of calls information gain
        # self.time3_overhead = 0
        # self.time3_overhead2 = 0
        # self.time3_overhead3 = 0
        # self.time3_overhead4 = 0
        # self.time4 = 0
        # self.time5 = 0
        # self.time6 = 0

    def fit(self, X, y):
        '''
        INPUT:
            - X: 2d numpy array
            - y: 1d numpy array
            - feature_names: numpy array of strings
        OUTPUT: None
        Build the decision tree.
        X is a 2 dimensional array with each column being a feature and each
        row a data point.
        y is a 1 dimensional array with each value being the corresponding
        label.
        feature_names is an optional list containing the names of each of the
        features.
        '''
        ## Convert to numpy array for faster implementation
        ## This improves efficiency significantly
        X = np.array(X)
        y = np.array(y)
        sort_index =  np.argsort(y)
        X = X[sort_index, :]
        y = y[sort_index]

        self.feature_importance_ = np.zeros(X.shape[1])
        self.feature_selected_ = np.zeros(X.shape[1])
        ## If max_features is not specified, use all features
        if self.max_features is None:
            self.max_features = X.shape[1]

        if self.feature_names is None or len(self.feature_names) != X.shape[1]:
            self.feature_names = np.arange(X.shape[1])
            #print("warning: feature_names has different dimension with X")

        # Create True/False array of whether the variable is categorical
        is_categorical = lambda x: isinstance(x, str)
        self.categorical = np.vectorize(is_categorical)(X[0])
        # start = now()
        self.root = self._build_tree(X, y, depth=0)
        self.feature_importance_ = self.feature_importance_ / np.sum(self.feature_importance_) ## Normalized feature
        # end = now()
        # print("build tree takes", end - start)
        return self.impurity_dict

    def _build_tree(self, X, y, depth):
        '''
        INPUT:
            - X: 2d numpy array
            - y: 1d numpy array
        OUTPUT:
            - TreeNode
        Recursively build the decision tree. Return the root node.
        '''

        node = TreeNode()
        # start = now()
        index, value, splits = self._choose_split_index(X, y)
        # end = now()
        # self.time1 += end - start
        if index is None or len(np.unique(y)) == 1 or depth >= self.max_depth or len(y) == 1:
            node.leaf = True
            node.classes = Counter(y)
            node.name = node.classes.most_common(1)[0][0]
        else:
            X1, y1, X2, y2 = splits
            node.column = index
            node.name = self.feature_names[index]
            node.value = value
            node.categorical = self.categorical[index]
            node.left = self._build_tree(X1, y1, depth + 1)
            node.right = self._build_tree(X2, y2, depth + 1)
        return node

    def _choose_split_index(self, X, y):
        '''
        INPUT:
            - X: 2d numpy array
            - y: 1d numpy array
        OUTPUT:
            - index: int (index of feature)
            - value: int/float/bool/str (value of feature)
            - splits: (2d array, 1d array, 2d array, 1d array)
        Determine which feature and value to split on. Return the index and
        value of the optimal split along with the split of the dataset.
        Return None, None, None if there is no split which improves information
        gain.
        Call the method like this:
        >>> index, value, splits = self._choose_split_index(X, y)
        >>> X1, y1, X2, y2 = splits
        '''

        max_gain, split_index, split_value, splits = 0, None, None, None

        ## Choose the features to use
        if X.shape[1] == self.max_features:
            # print("No resamples")
            random_features = range(self.max_features)
        else:
            # print("Resamples")
            random_features = np.random.choice(range(X.shape[1]),self.max_features)

        ## Make splits
        for i in random_features:
            if self.feature_threshold is None:
                ## No user specified threshold provided, use threshold from data
                if self.categorical[i]:
                    values = np.unique(X[:, i])
                    if i not in self.feature_threshold_out_:
                        self.feature_threshold_out_[i] = values
                else:
                    if self.continous_variable_splits.lower() == "percentile".lower():
                        percentiles_ = np.arange(10, 100, 10)
                        values = np.percentile(X[:,i], percentiles_)
                    else:
                        values = np.unique(X[:,i])
                    if i not in self.feature_threshold_out_:
                        self.feature_threshold_out_[i] = list(values)
            else:
                ## Use user specified thresholds
                values = self.feature_threshold[i]
            if len(values) < 2:
                continue
            # max_gain, split_index, split_value, splits = self._search_max_gain(X, y, i, values, max_gain, split_index,
            #                                                                    split_value, splits)
            max_gain, split_index, split_value, splits = search_maxgain_mask(X, y, values, i, max_gain,
                                                                             split_index, split_value, splits,
                                                                             categorical=self.categorical[i],
                                                                             crit=self.impurity_criterion)
        if split_index is not None:
            # print(split_index, max_gain)
            self.feature_importance_[split_index] += max_gain * len(y)
            self.feature_selected_[split_index] += 1

        return split_index, split_value, splits

    def _search_max_gain(self, X, y, i, values, max_gain, split_index, split_value, splits):
        '''
        Search for max gain
        :param X:
        :param y:
        :param i:
        :param values:
        :param max_gain:
        :param split_index:
        :param split_value:
        :param splits:
        :return:
        '''
        start = now()
        for val in values:
            X1, y1, X2, y2 = self._make_split(X, y, i, val)
            # start_1 = now()
            gain = self._information_gain(y, y1, y2)
            # end_1 = now()
            # self.time3_overhead += end_1 - start_1
            if gain > max_gain and len(y1) > 0 and len(y2) > 0:
                max_gain = gain
                split_index, split_value = i, val
                splits = X1, y1, X2, y2
        end = now()
        self.time1 += end - start
        return max_gain, split_index, split_value, splits

    def _make_split(self, X, y, split_index, split_value):
        '''
        INPUT:
            - X: 2d numpy array
            - y: 1d numpy array
            - split_index: int (index of feature)
            - split_value: int/float/bool/str (value of feature)
        OUTPUT:
            - X1: 2d numpy array (feature matrix for subset 1)
            - y1: 1d numpy array (labels for subset 1)
            - X2: 2d numpy array (feature matrix for subset 2)
            - y2: 1d numpy array (labels for subset 2)
        Return the two subsets of the dataset achieved by the given feature and
        value to split on.
        Call the method like this:
        >>> X1, y1, X2, y2 = self._make_split(X, y, split_index, split_value)
        X1, y1 is a subset of the data.
        X2, y2 is the other subset of the data.
        '''
        # self.n2 += 1
        # start = now()
        split_col = X[:, split_index]
        if self.categorical[split_index]:
            mask = split_col == split_value
        else:
            mask = split_col < split_value
        result = X[mask, :], y[mask], X[~mask, :], y[~mask]
        # end = now()
        # self.time2 += end - start
        return result

    def _entropy(self, y):
        '''
        INPUT:
            - y: 1d numpy array
        OUTPUT:
            - float
        Return the entropy of the array y.
        '''
        n_y = float(len(y))
        cbincount = np.bincount(y)
        total = 0
        for val in cbincount:
            prob = val / n_y
            total += prob * math.log(prob)
        return -total

    def _gini(self, y):
        n_y = float(len(y))
        cbincount = np.bincount(y)
        total = 0
        for val in cbincount:
            total += (val/n_y) ** 2
        return 1 - total

    def _information_gain(self, y, y1, y2):
        '''
        INPUT:
            - y: 1d numpy array
            - y1: 1d numpy array (labels for subset 1)
            - y2: 1d numpy array (labels for subset 2)
        OUTPUT:
            - float
        Return the information gain of making the given split.
        Use self.impurity_criterion(y) rather than calling _entropy or _gini
        directly.
        '''
        total = self.impurity_criterion(y)
        for y_split in (y1, y2):
            ent = self.impurity_criterion(y_split)
            total -= len(y_split) * ent / float(len(y))
        return total

    def predict(self, X):
        '''
        INPUT:
            - X: 2d numpy array
        OUTPUT:
            - y: 1d numpy array
        Return an array of predictions for the feature matrix X.
        '''

        return np.array([self.root.predict_one(row) for row in X])

    def score(self, X, y):
        '''
        Return the accuracy of the Random Forest for the given test data and
        labels.
        '''
        y_pred = self.predict(X)
        n_right = np.sum(y == y_pred)
        n_total = len(y)
        return float(n_right)/n_total

    def __str__(self):
        '''
        Return string representation of the Decision Tree.
        '''
        return str(self.root)

    def __eq__(self, other):
        return self.root == other.root






## Self constructed logistic regression
## The solver is gradient descent
import numpy as np

class GradientDescent(object):
    """Preform the gradient descent optimization algorithm for an arbitrary
    cost function.
    """
    def __init__(self, cost, gradient, predict_func,
                 fit_intercept=False,
                 alpha=0.01,
                 num_iterations=10000,
                 lamb=0):
        """Initialize the instance attributes of a GradientDescent object.

        Parameters
        ----------
        cost: The cost function to be minimized.
        gradient: The gradient of the cost function.
        predict_func: A function to make predictions after the optimizaiton has
            converged.
        alpha: The learning rate.
        num_iterations: Number of iterations to use in the descent.

        Returns
        -------
        self: The initialized GradientDescent object.
        """
        # Initialize coefficients in run method once you know how many features
        # you have.
        self.coeffs = None
        self.cost = cost
        self.gradient = gradient
        self.predict_func = predict_func
        self.fit_intercept = fit_intercept
        self.alpha = alpha
        self.num_iterations = num_iterations
        self.lamb = lamb
        self.cost_hist = []
    def fit(self, X, y):
        """Run the gradient descent algorithm for num_iterations repititions.

        Parameters
        ----------
        X: A two dimenstional numpy array.  The training data for the
            optimization.
        y: A one dimenstional numpy array.  The training response for the
            optimization.

        Returns
        -------
        self:  The fit GradientDescent object.
        """
        if self.fit_intercept:
            X_new = self.X_add_ones(X)
            self.coeffs = np.ones((X_new.shape[1], 1))
            for i in range(self.num_iterations):
                grad = self.gradient(X_new,y,self.coeffs,lamb=self.lamb)
                if i%100 == 0:
                    self.cost_hist.append(self.cost(X_new,y,self.coeffs,lamb=self.lamb))
                self.coeffs = self.coeffs - (self.alpha * grad)
        else:
            self.coeffs = np.ones((X.shape[1], 1))
            for i in range(self.num_iterations):
                grad = self.gradient(X,y,self.coeffs,lamb=self.lamb)
                if i%100 == 0:
                    self.cost_hist.append(self.cost(X,y,self.coeffs,lamb=self.lamb))
                self.coeffs = self.coeffs - (self.alpha * grad)

    def predict(self, X):
        """Call self.predict_func to return predictions.

        Parameters
        ----------
        X: Data to make predictions on.

        Returns
        -------
        preds: A one dimensional numpy array of predictions.
        """
        return self.predict_func(X,self.coeffs)
    def X_add_ones(self,X):
        m,n = X.shape
        X_new = np.ones((m,n+1))
        X_new[:,0:n] = X
        return X_new

class Logistic_Regression_JL(object):

    def __init__(self,alpha = 0.001,fit_intercept=False,lamb = 0):
        self.coefs_ = None
        self.intercept = None
        self.X = None
        self.alpha = alpha
        self.fit_intercept = fit_intercept ## Intercept
        self.lamb = lamb ## Regularization strength

    def sigmoid(self,X,coefs_):
        return 1.0 / (1 + np.exp(-np.dot(X,coefs_)))

    def cost_logistic(self,X,y,coefs_,lamb=0):
        n = y.shape[0]
        hx = self.sigmoid(X,coefs_)
        cost = -np.sum(y * np.log(hx) + (1 - y) * np.log( 1 - hx))
        ridge_cost = np.sum(coefs_*coefs_)
        return cost + lamb * ridge_cost

    def gradient(self,X,y,coefs_,lamb=0):
        hx = self.sigmoid(X,coefs_)
        y_reshape = y.reshape(y.shape[0], 1)
        ridge_grd = 2 * coefs_
        return np.dot((hx - y_reshape).T, X).T + lamb * ridge_grd

    def predict_proba(self,X,coefs_):
        if self.fit_intercept:
            X_new = self.X_add_ones(X)
            return self.sigmoid(X_new,coefs_)
        else:
            return self.sigmoid(X,coefs_)
    def predict(self,X,threshold = 0.5):
        predict = self.predict_proba(X,self.coefs_)
        predict[predict > threshold] = 1
        predict[predict <= threshold] = 0
        return predict
    def fit(self,X, y, alpha_=None):
        if alpha_ is None:
            alpha_ = self.alpha
        gd = GradientDescent(self.cost_logistic, self.gradient, self.predict, alpha=alpha_
                             ,fit_intercept=self.fit_intercept,lamb=self.lamb)
        gd.fit(X, y)
        self.coefs_ = gd.coeffs
    def score(self,X,y):
        y_pred = self.predict(X)
        y = y.reshape(y_pred.shape)
#        print(y.shape,y_pred.shape,np.sum(y_pred == y))
        return np.sum(y_pred == y) / float(y.shape[0])
    def X_add_ones(self,X):
        m,n = X.shape
        X_new = np.ones((m,n+1))
        X_new[:,0:n] = X
        return X_new


import numpy as np
from numpy.linalg import lstsq
from scipy.optimize import nnls
class NMF:
    def __init__(self, k=10, maxiter = 10):
        self.k = k
        self.maxiter = maxiter
        self.V = None
        self.W = None
        self.H = None

    def init_matrix_(self):
        n,m = self.V.shape
        self.W = np.random.rand(n, self.k)
        self.H = np.random.rand(self.k, m)

    def fit(self, V):
        self.V = V
        self.init_matrix_()
        cost = []
        for iter in xrange(self.maxiter):
            self.H, r1 = lstsq(self.W, self.V)[0:2]
            self.H = np.clip(self.H, a_min=0, a_max= None)
            TEM, r2 = lstsq(self.H.T, self.V.T)[0:2]
            self.W = TEM.T
            self.W = np.clip(self.W, a_min=0, a_max= None)
            cost.append(np.sum((self.V - np.dot(self.W, self.H))**2))
        return self.W, self.H , cost

    # def fit2(self, V):
    #     self.V = V
    #     self.init_matrix_()
    #     cost = []
    #     for iter in xrange(self.maxiter):
    #         self.H, r1 = nnls(self.W, self.V)
    #         TEM, r2 = nnls(self.H.T, self.V.T)
    #         self.W = TEM.T
    #         cost.append(np.sum((self.V - np.dot(self.W, self.H))**2))
    #     return self.W, self.H , cost


import numpy as np
from sklearn.base import ClassifierMixin

def euclidean_distance(x1,x2):
    ## x1 is n1 x m and x2 is m x n2
    ## return array of shape n1 x n2
    if isinstance(x1,list) and isinstance(x2,list):
        x1, x2 = np.array(x1), np.array(x2)
        return np.sum(np.square(x1-x2))
    x1, x2 = np.array(x1), np.array(x2)
    if x1.shape[1] != x2.shape[1]:
        return "Error : array x1 and x2 should have dimesion same in axis=1"
    n1,m = x1.shape
    n2,m = x2.shape
    x1_square = np.sum(np.square(x1), axis=1).reshape((n1,1))
    x2_square = np.sum(np.square(x2.T), axis=0).reshape((1,n2))
    x1_x2 = np.dot(x1,x2.T)
#    print(x1_x2.shape,x1_square.shape,x2_square.shape)
    return -x1_x2 * 2 + x1_square + x2_square

def cosine_distance(x1,x2):
    ## x1 is n1 x m and x2 is m x n2
    ## return array of shape n1 x n2
    if isinstance(x1,list) and isinstance(x2,list):
        x1, x2 = np.array(x1), np.array(x2)
        return 1 - np.dot(x1,x2.T)/np.sqrt(np.sum(x1**2))/np.sqrt(np.sum(x2**2))
    x1, x2 = np.array(x1), np.array(x2)
    if x1.shape[1] != x2.shape[1]:
        return "Error : array x1 and x2 should have dimesion same in axis=1"
    n1,m = x1.shape
    n2,m = x2.shape
    x1_sqrt = np.sqrt(np.sum(np.square(x1), axis=1).reshape((n1,1)))
    x2_sqrt = np.sqrt(np.sum(np.square(x2.T), axis=0).reshape((1,n2)))
    x1_x2 = np.dot(x1,x2.T)
#    print(x1_x2.shape,x1_square.shape,x2_square.shape)
    return 1 - x1_x2/x1_sqrt/x2_sqrt



class KNearestNeighbors():
    def __init__(self, k=1, distance=euclidean_distance):
        self.k = k
        self.X = None
        self.y = None
        self.n_rows_ = None
        self.dist = distance
    def fit(self,X,y):
        self.X = X
        self.y = y
        self.n_rows_ = X.shape[0]
    def predict(self,X_new):
        n_new = X_new.shape[0]
#        print(X_new.shape,self.X.shape)
        dist_mat = euclidean_distance(X_new,self.X)
        sort_index = np.argsort(dist_mat,axis = 1)
        y_pred = np.zeros((n_new,1))
        for i in xrange(n_new):
            y_pred[i] = np.mean(self.y[sort_index[i,0:self.k]])
        # print("Distant Matrix")
        # print(dist_mat)
        # print(self.y[sort_index[0,0:self.k]])
        # print(np.mean(self.y[sort_index[0,0:self.k]]))
        # print(sort_index.shape)
        # print(y_pred)
        y_pred = (y_pred > 0.5).astype(int)
        return y_pred
    def score(self,X,y_true):
        n = len(y_true)
        y_pred = self.predict(X)
        y_true,y_pred = np.array(y_true).reshape((n,1)),np.array(y_pred).reshape((n,1))
        return float(len(y_true[y_true - y_pred == 0]))/len(y_true)




# from tree import DecisionTreeClassifier
## Reference for packages with cython and python
## https://groups.google.com/forum/#!msg/cython-users/6trL0V1bLx4/7bxhj0xCK50J
## https://github.com/cython/cython/wiki/PackageHierarchy
## https://github.com/scikit-learn/scikit-learn/blob/master/sklearn/tree/setup.py


__all__ = [".", 'tree', 'ctree', 'Kmeans', "cyfun"]



import numpy as np
import matplotlib.pyplot as plt
from sklearn.metrics import roc_curve, auc
from scipy import interp
from sklearn.cross_validation import KFold
from sklearn.preprocessing import StandardScaler


def plot_roc(X, y, clf_class, **kwargs):
    scaler = StandardScaler()
    X = scaler.fit_transform(X)
    kf = KFold(len(y), n_folds=5, shuffle=True)
    y_prob = np.zeros((len(y),2))
    mean_tpr = 0.0
    mean_fpr = np.linspace(0, 1, 100)
    all_tpr = []
    for i, (train_index, test_index) in enumerate(kf):
        X_train, X_test = X[train_index], X[test_index]
        y_train = y[train_index]
        clf = clf_class(**kwargs)
        clf.fit(X_train,y_train)
        # Predict probabilities, not classes
        y_prob[test_index] = clf.predict_proba(X_test)
        fpr, tpr, thresholds = roc_curve(y[test_index], y_prob[test_index, 1])
        mean_tpr += interp(mean_fpr, fpr, tpr)
        mean_tpr[0] = 0.0
        roc_auc = auc(fpr, tpr)
        plt.plot(fpr, tpr, lw=1, label='ROC fold %d (area = %0.2f)' % (i, roc_auc))
    mean_tpr /= len(kf)
    mean_tpr[-1] = 1.0
    mean_auc = auc(mean_fpr, mean_tpr)
    plt.plot(mean_fpr, mean_tpr, 'k--',label='Mean ROC (area = %0.2f)' % mean_auc, lw=2)
    
    plt.plot([0, 1], [0, 1], '--', color=(0.6, 0.6, 0.6), label='Random')
    plt.xlim([-0.05, 1.05])
    plt.ylim([-0.05, 1.05])
    plt.xlabel('False Positive Rate')
    plt.ylabel('True Positive Rate')
    plt.title('Receiver operating characteristic')
    plt.legend(loc="lower right")
    plt.show()


import numpy as np
from sklearn import metrics
class Kmeans:
    def __init__(self, K=3, max_iter=100, tol = 1e-4):
        self.X = None
        self.K = K
        self.centroids = None
        self.clusters = None
        self.max_iter_ = max_iter
        self.tol_ = tol

    def init_centroid_(self):
        if self.X is None:
            return "X is not initialized yet"
        m,n = self.X.shape
        choice_index = np.arange(m)
        choice_index_k = np.random.choice(choice_index, size=self.K, replace=False)
        #print(self.X[choice_index_k,:].shape)
        self.centroids = self.X[choice_index_k,:]

    def distance_(self,distance_fun = "square"):
        m, n = self.X.shape
        D = np.zeros((m, self.K))
        if distance_fun=="square":
            for i in xrange(self.K):
                D[:,i] = np.sum((self.X - self.centroids[i,:])**2, axis=1)
        return D

    def update_cluster(self):
        self.clusters = np.argmin(self.distance_(),axis=1)

    def update_centroids(self):
        centroid_old = np.copy(self.centroids)
        for i in xrange(self.K):
            self.centroids[i,:] = np.mean(self.X[self.clusters == i,:],axis = 0)
        dif = np.sum((centroid_old - self.centroids)**2)
        return dif

    def fit(self,X, K=3):
        self.K = K
        self.X = X
        self.init_centroid_()
        iter = 0
        while iter < self.max_iter_:
            self.update_cluster()
            dif = self.update_centroids()
           # print("Updating iteratio ", iter,dif)
            if (dif < self.tol_):
                break
            iter += 1
        return self.clusters

    def elbow_dist(self):
        dist = 0
        for i in xrange(self.K):
            cluster_mask = (self.clusters == i)
            dist = dist + np.sum((self.X[cluster_mask,:] - self.centroids[i,:]) ** 2)
        return dist

    def elbow_fit(self,X,K_max = 10):
        elbow_dist_list = []
        for k in xrange(1,K_max + 1):
            self.K = k
            self.fit(X, K=k)
            elbow_dist_list.append(self.elbow_dist())
        return elbow_dist_list

    def Silhouette_fit(self,X,K_max = 10):
        Silhouette_dist_list = []
        for k in xrange(2,K_max + 1):
            self.K = k
            clusters = self.fit(X, K=k)
            Silhouette_dist_list.append(metrics.silhouette_score(X, clusters, metric='euclidean'))
        return Silhouette_dist_list



from setuptools import setup
from setuptools.extension import Extension
try:
    from Cython.Build import cythonize
except ImportError:
    use_cython = False
else:
    use_cython = True

ext_modules = []
if use_cython:
    ext_modules += cythonize('houstonj2013_ml/cyfun.pyx')
else:
    ext_modules += [Extension('houstonj2013_ml.cyfun',
                              ['houstonj2013_ml/cyfun.c'])]

setup(name='houstonj2013_ml',
      ext_modules=ext_modules,
      version='1.0',
      description='houstonj2013 machine learning modules',
      url='http://github.com/HoustonJ2013/HoustonJ2013_ml',
      author='Jingbo Liu',
      author_email='jingbo.liu2013@gmail.com',
      license='selfie',
      packages=['houstonj2013_ml'],
      zip_safe=False
      )

